package fr.natan.akkastreamfileprocessingapi.controller;

import fr.natan.akkastreamfileprocessingapi.futurecompleteness.CompletableFutureResult;
import fr.natan.akkastreamfileprocessingapi.models.Models;
import fr.natan.akkastreamfileprocessingapi.service.AkkaStreamFileProcessing;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import play.api.libs.json.Json;
import scala.collection.IndexedSeq;
import scala.concurrent.Future;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;


@SuppressWarnings("SpellCheckingInspection")
@RestController
public class MovieController {

    private final AkkaStreamFileProcessing akkaStreamFilesProcessing;
    private final CompletableFutureResult completableFutureResult = new CompletableFutureResult();
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    public MovieController(AkkaStreamFileProcessing akkaStreamFilesProcessing) {
        this.akkaStreamFilesProcessing = akkaStreamFilesProcessing;
    }

    @RequestMapping(value = "/persons/id/{personID}", method = RequestMethod.GET)
    private ResponseEntity<String> getPersonByID(@PathVariable(name = "personID") String personID) throws ExecutionException, InterruptedException {
        Future<Models.Person> futurePerson = akkaStreamFilesProcessing.getPersonById(personID);
        CompletableFuture<Models.Person> completableFuture = completableFutureResult.buildcompletableFuture(futurePerson);
        while (!completableFuture.isDone()){
           logger.info("IS PROCESSING...");
            Thread.sleep(5000);
        }
        Models.Person person = completableFuture.get();
        return new ResponseEntity<>(Json.toJson(person, person.personFormat()).toString(), HttpStatus.OK);
    }

    @RequestMapping(value = "/persons/name/{primaryName}", method = RequestMethod.GET)
    private ResponseEntity<String> getPersonByName(@PathVariable(name = "primaryName") String primaryName)
            throws ExecutionException, InterruptedException {

        Future<Models.Person> personFuture = akkaStreamFilesProcessing.getPersonByName(primaryName);
        CompletableFuture<Models.Person> personCompletableFuture = completableFutureResult.buildcompletableFuture(personFuture);
        while (!personCompletableFuture.isDone()){
            logger.info("IS PROCESSING...");
            Thread.sleep(5000);
        }
        Models.Person person = personCompletableFuture.get();
        return new ResponseEntity<>(Json.toJson(person, person.personFormat()).toString(), HttpStatus.OK);
    }

    @RequestMapping(value = "/tvseries/id/{tvSerieId}", method = RequestMethod.GET)
    private ResponseEntity<String> getTvSerieByID(@PathVariable(name = "tvSerieId") String tvSerieID) throws ExecutionException, InterruptedException {
        Future<Models.TvSerie> tvSerieFuture = akkaStreamFilesProcessing.getTvSerieById(tvSerieID);
        CompletableFuture<Models.TvSerie> tvSerieCompletableFuture = completableFutureResult.buildcompletableFuture(tvSerieFuture);
        while (!tvSerieCompletableFuture.isDone()){
            logger.info("IS PROCESSING...");
            Thread.sleep(5000);
        }
        Models.TvSerie tvSerie = tvSerieCompletableFuture.get();
        return new ResponseEntity<>(Json.toJson(tvSerie, tvSerie.tvSerieFormat()).toString(), HttpStatus.OK);
    }

    @RequestMapping(value = "/tvseries/title/{tvseriePrimaryTitle}", method = RequestMethod.GET)
    private ResponseEntity<IndexedSeq<Models.TvSerie>> getTvserieByPrimaryTitle(
            @PathVariable(name = "tvseriePrimaryTitle") String tvseriePrimaryTitle) {

        Future<IndexedSeq<Models.TvSerie>> listFuture = akkaStreamFilesProcessing.getTvSerieByPrimaryTitle(tvseriePrimaryTitle);
        IndexedSeq<Models.TvSerie> indexedSeq = listFuture.value().get().get();

        return new ResponseEntity<>(indexedSeq, HttpStatus.OK);
    }

    @RequestMapping(value = "/persons/tvseries/title/{tvSerieTitle}", method = RequestMethod.GET)
    private Future<IndexedSeq<Models.Person>> getPersonsForTvSerie(@PathVariable(name = "tvSerieTitle") String tvSerieTitle) {
        return akkaStreamFilesProcessing.getTeamOfPersonsForTvSerie(tvSerieTitle);
    }

    @RequestMapping(value = "/persons", method = RequestMethod.GET)
    private ResponseEntity<String> getAllPersons() {
        akkaStreamFilesProcessing.getAllPersons();
        return new ResponseEntity<>("is running", HttpStatus.OK);
    }

    @RequestMapping(value = "/tvseries", method = RequestMethod.GET)
    private ResponseEntity<String> getAllvSeries() {
        akkaStreamFilesProcessing.getAllTvSeries();
        return new ResponseEntity<>("is running", HttpStatus.OK);
    }
}
